import logging

from django.conf import settings
from django.core.mail.backends.base import BaseEmailBackend
from django.utils.importlib import import_module

from .utils import translate

logger = logging.getLogger(__name__)


class EmailBackend(BaseEmailBackend):

    def __init__(self, backend=None, *args, **kwargs):
        if backend is None:
            backend = getattr(settings, 'DEMAIL_BACKEND',
                              'django.core.mail.backends.console.EmailBackend')
        logger.debug('EMAIL_BACKEND = %r', backend)

        if isinstance(backend, basestring):
            module, cls = backend.rsplit('.', 1)
            backend = getattr(import_module(module), cls)

        self.backend = backend()

    def send_messages(self, email_messages):
        return self.backend.send_messages(map(translate, email_messages))



"""
This file demonstrates writing tests using the unittest module. These will pass
when you run "manage.py test".

Replace this with more appropriate tests for your application.
"""

from django.conf import settings
from django.core import mail
from django.test import TestCase
from django.test.utils import override_settings


@override_settings(EMAIL_BACKEND='demail.backends.EmailBackend')
class DelegateEmailTest(TestCase):
    """
    It is necessary to override the ``EMAIL_BACKEND`` setting here, regardless
    of the value in the test_app, because the Django test runner will always
    force it to be ``'django.core.mail.backends.locmem.EmailBackend'`` when it
    sets up the testing environment.

    See http://stackoverflow.com/a/15053970 for details.
    """

    def setUp(self):
        self.recipient_list = [
            'gary@touch.asn.au',
            'john.reynolds@touchtechnology.com.au',
        ]

    @override_settings(
        EMAIL_BACKEND='django.core.mail.backends.locmem.EmailBackend')
    def test_pass_through(self):
        mail.send_mail(u'Subject',
                       u'',
                       settings.DEFAULT_FROM_EMAIL,
                       self.recipient_list)

        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].subject, 'Subject')
        self.assertEqual(mail.outbox[0].to, self.recipient_list)

    @override_settings(DEMAIL_ALLOWED_DOMAINS=('touchtechnology.com.au',))
    def test_allowed_domains(self):
        mail.send_mail(u'Subject',
                       u'',
                       settings.DEFAULT_FROM_EMAIL,
                       self.recipient_list)

        recipients = [
            'gary=touch.asn.au@example.com',
            'john.reynolds@touchtechnology.com.au',
        ]

        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].subject, 'Subject')
        self.assertEqual(mail.outbox[0].to, recipients)

    @override_settings(DEMAIL_ALLOWED_RECIPIENTS=('gary@touch.asn.au',))
    def test_allowed_recipients(self):
        mail.send_mail(u'Subject',
                       u'',
                       settings.DEFAULT_FROM_EMAIL,
                       self.recipient_list)

        recipients = [
            'gary@touch.asn.au',
            'john.reynolds=touchtechnology.com.au@example.com',
        ]

        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].subject, 'Subject')
        self.assertEqual(mail.outbox[0].to, recipients)

    @override_settings(DEMAIL_REWRITE_DOMAIN='test.example.com')
    def test_rewrite_domain(self):
        mail.send_mail(u'Subject',
                       u'',
                       settings.DEFAULT_FROM_EMAIL,
                       self.recipient_list)

        recipients = [
            'gary=touch.asn.au@test.example.com',
            'john.reynolds=touchtechnology.com.au@test.example.com',
        ]

        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].subject, 'Subject')
        self.assertEqual(mail.outbox[0].to, recipients)

    @override_settings(DEMAIL_REWRITE_RECIPIENT='test@example.com')
    def test_rewrite_recipient(self):
        mail.send_mail(u'Subject',
                       u'',
                       settings.DEFAULT_FROM_EMAIL,
                       self.recipient_list)

        recipients = [
            'test+gary=touch.asn.au@example.com',
            'test+john.reynolds=touchtechnology.com.au@example.com',
        ]

        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].subject, 'Subject')
        self.assertEqual(mail.outbox[0].to, recipients)

    # @override_settings(
    #     DEMAIL_BACKEND='django.core.mail.backends.console.EmailBackend')
    # def test_backend(self):
    #     mail.send_mail(u'Subject',
    #                    u'',
    #                    settings.DEFAULT_FROM_EMAIL,
    #                    self.recipient_list)



import logging
import re

from django.conf import settings

logger = logging.getLogger(__name__)

TRANS = {
    r'@': '=',
    r'\+': '-',
}


def translate(message):
    """
    Map the ``rewrite_recipient_address`` function to each addressee of this
    message.
    """
    allowed_recipients = getattr(settings, 'DEMAIL_ALLOWED_RECIPIENTS', ())
    allowed_domains = getattr(settings, 'DEMAIL_ALLOWED_DOMAINS', ())
    rewrite_domain = getattr(settings, 'DEMAIL_REWRITE_DOMAIN', 'example.com')
    rewrite_recipient = getattr(settings, 'DEMAIL_REWRITE_RECIPIENT', None)

    logger.debug('ALLOWED_RECIPIENTS = %r', allowed_recipients)
    logger.debug('ALLOWED_DOMAINS = %r', allowed_domains)
    logger.debug('REWRITE_DOMAIN = %r', rewrite_domain)
    logger.debug('REWRITE_RECIPIENT = %r', rewrite_recipient)

    rewrite = lambda addr: rewrite_recipient_address(addr,
                                                     allowed_domains,
                                                     allowed_recipients,
                                                     rewrite_domain,
                                                     rewrite_recipient)

    message.to = map(rewrite, message.to)
    message.cc = map(rewrite, message.cc)
    message.bcc = map(rewrite, message.bcc)

    return message


def rewrite_recipient_address(addr, allowed_domains, allowed_recipients,
                              rewrite_domain, rewrite_recipient):
    """
    Given a destination email address, determine if it is acceptable to
    deliver email directly to this address. If it is not then we rewrite the
    value so that delivery will go to an alternative mailbox.

    The value of ``email`` is assumed to be correct - the Django reference
    implementation ``django.core.mail.backends.smtp.EmailBackend`` does not
    perform any validation, so nor shall we.
    """
    logger.debug('processing addr %r', addr)

    # if addr is specifically declared deliverable, let it straight through
    if addr in allowed_recipients:
        logger.debug('%r in ALLOWED_RECIPIENTS', addr)
        return addr

    # if the domain part of the addr is declared a deliverable mail exchange,
    # also let it through.
    localpart, domain = addr.split(u'@', 1)
    if domain in allowed_domains:
        logger.debug("%r matches ALLOWED_DOMAIN '%s'", addr, domain)
        return addr

    localpart = addr
    for pat, repl in TRANS.items():
        localpart = re.sub(pat, repl, localpart)

    if rewrite_recipient:
        mailbox, domain = rewrite_recipient.split(u'@', 1)
        output = '{0}+{1}@{2}'.format(mailbox, localpart, domain)
    else:
        output = '{0}@{1}'.format(localpart, rewrite_domain)
    logger.warning('rewriting %r -> %r', addr, output)
    return output



from os.path import dirname, join as basejoin, realpath

join = lambda *args: realpath(basejoin(*args))

PROJECT_DIR = join(dirname(__file__), '..')
DEBUG = True
TEMPLATE_DEBUG = DEBUG

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
    }
}

TIME_ZONE = 'Australia/Sydney'
LANGUAGE_CODE = 'en-AU'

USE_TZ = True
STATIC_URL = '/static/'

SECRET_KEY = ')l+649%yw_so!5twpu1-e&amp;b(2ldpswad9_*vdc8t@de4)h@n4j'

ROOT_URLCONF = 'test_app.urls'
WSGI_APPLICATION = 'test_app.wsgi.application'

INSTALLED_APPS = (
    'demail',
)

# Critical that the DEMAIL_BACKEND defaults to LocMem as this is what the test
# environment will expect. Inspecting the ``mail.outbox`` in tests will fail
# if the test case uses a different backend.
DEMAIL_BACKEND = 'django.core.mail.backends.locmem.EmailBackend'

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        },
    },
    'formatters': {
        'verbose': {
            'format': '%(levelname)-10s %(name)-30s %(asctime)-27s '
                      '%(message)s',
        },
        'simple': {
            'format': '%(levelname)s %(message)s',
        },
    },
    'handlers': {
        'null': {
            'class': 'django.utils.log.NullHandler',
        },
        'console': {
            'class': 'logging.StreamHandler',
            'formatter': 'verbose',
        },
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        },
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
        'demail': {
            'handlers': ['console'],
            'level': 'ERROR',
        },
    },
}



# This file mainly exists to allow python setup.py test to work.

import os
import sys
os.environ['DJANGO_SETTINGS_MODULE'] = 'test_app.settings'

test_dir = os.path.dirname(__file__)
sys.path.insert(0, test_dir)

from django.test.utils import get_runner
from django.conf import settings


def runtests():
    TestRunner = get_runner(settings)
    test_runner = TestRunner(verbosity=2, interactive=True)
    failures = test_runner.run_tests(['demail'])
    sys.exit(failures)

if __name__ == '__main__':
    runtests()



from setuptools import setup, find_packages

version = "0.2"

setup(
    name='django-demail',
    version=version,
    packages=find_packages(exclude=['test_app']),
    author='Gary Reynolds',
    author_email='gary@touch.asn.au',
    description='Delegating email backend.',
    url='https://github.com/goodtune/django-demail',
    install_requires=[
        'Django',
    ],
    test_suite='test_app.runtests.runtests',
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries',
    ],
)
